/////////////////////////////////////////////////////////////////////////////////
// file: analysis_main
// 
// Description: main class controlling analysis code for parton level 
//              single top analysis.
//
// Author: Reinhard Schwienhorst
//
// Date: 04/07/2005
//
// Modifications:
//
/////////////////////////////////////////////////////////////////////////////////
#include "single_top_analysis/analysis_main.hpp"
#include <iostream>
#include <string>
#include <fstream>

#include <TStopwatch.h>
#include <TTreeIndex.h>
#include <TSystem.h>

#include "single_top_analysis/utils.hpp"
//

using namespace std;

// needed to run with compiled Root:
ClassImp(analysis_main)


//
// this is where the action is:
void analysis_main::loop()
{
  // book histograms
  SetDirectory (_histoFile);
  CutListProcessor::BookHistos();
  //
  // now start the loop over events 
  // first over the root file without gluons:
  bool limitEvents= (_numEvents>0);

  // start the stopwatch
  TStopwatch w;
  w.Start();
  
  int iEvt=GetNextEvent();
  int nevents=0;
  // either loop until we are out of events or until we have reached
  // the limit specified on the command line
  //cout<<"Beginning loop over events, _numEvent is "<<_numEvents<<endl;
  while(iEvt != -1 && (!limitEvents || iEvt<_numEvents)) {  
    nevents++;
    if(iEvt%20000==0) cout<<"Processing event "<<iEvt<<endl;

    //evtNum=iEvt;
    //
    //
    // this is from CutListProcessor:
    // now fill the histograms
    //cout<<"calling apply"<<endl;
    CutListProcessor::Apply(this);
    //
    // and read the next event
    //cout<<"Getting next event"<<endl;
    iEvt=GetNextEvent();
  } // end of loop!

  cout<<endl<<"Done processing "<<nevents<<" events."<<endl<<endl;
  // stop the timer and print out timing statistics
  w.Stop();
  Double_t time_spent=w.RealTime();
  Double_t time_event=time_spent/(Double_t)nevents;

  cout<<"Timing info:"<<endl;
  cout<<"  Real time (s):      "<<time_spent<<endl;
  cout<<"  Time per event (s): "<<time_event<<endl;
  cout<<"  Events/second       "<<1./time_event<<endl;
  cout<<endl;


  // writing histogram file and close it
  writeHistogramFile();
  closeHistogramFile();
  

} 

/////////////////////////////////////////////////////////////////////
// constructor:
analysis_main::analysis_main(): CutListProcessor("cutter")
			  , _histoFile(NULL),_numEvents(0),_histoFileName("dummy.root")
{
}

////////////////////////////////////////////////////////////////////////
// destructor:
analysis_main::~analysis_main()
{
}

////////////////////////////////////////////////////////////////////////////////////////////
//
// function to parse the command line and extract
// the name of the top root file and trigger root file.
Int_t analysis_main::parsecmdline(int argc, char **argv,TChain *coch,TChain *evch,TChain *trch)
{
  if(argc<3) {
    cout<<"**********************************************************************"<<endl;
    cout<<"* Program: analysis_main                                             *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"* Purpose: analyze a parton level single top root file.              *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"* Usage: analysis_main -inlist filename                              *"<<endl;
    cout<<"*  You need to specify a root file or list of files                  *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"*  -infile file    - load a single root file.                        *"<<endl;
    cout<<"*  -inlist file    - open a file with a list of root files.          *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"*  -num #          - number of events to process.                    *"<<endl;
    cout<<"*  -offSet #       - start processing not at 0 but here.             *"<<endl;
    cout<<"*  -hfile file     - filename for output histograms.                 *"<<endl;
    cout<<"*  -mtop           - tarGet top mass for best jet algorithm.         *"<<endl;
    cout<<"*  -lepton #       - only process events with this lepton flavor.    *"<<endl;
    cout<<"*  -norm #         - Multiply histogram weight by #.                 *"<<endl;
    cout<<"*                                                                    *"<<endl;
    cout<<"**********************************************************************"<<endl;
    return 1;
  }

  _debugLevel=20;
 
  cout<<"analysis_main: Parsing command line."<<endl;

  // check that we have valid chain pointers:
  if(coch==0) {
    cout<<"<analysis_main::parsecmdline> No valid input Collection Chain."<<endl;
    return -1;
  }

  // check that we have valid chain pointers:
  if(evch==0) {
    cout<<"<analysis_main::parsecmdline> No valid input Event Chain."<<endl;
    return -1;
  }

  // check that we have valid chain pointers:
  if(trch==0) {
    cout<<"<analysis_main::parsecmdline> No valid input Truth Chain."<<endl;
    //cout<<"This is ok for now..."<<endl;
    return -1;
  }

  // default histogram file name:
  std::string histoFileName="Histograms.root";


  CollectionTree* treeCollection=0;
  EventTree* treeEventTree=0;
  TruthTree* treeTruthTree=0;

  int evtOffSet=0;

  // Variables for IO
  int iLOList = 0; // Index of pointer to file-list name in arguments
  int iLOFile = 0; // Index of pointer to file name in arguements
  bool useLOFile = false;
  bool useLOList = false;

  //
  // save the current path to get back to it later
  string cwd=gSystem->pwd();
  
  // Parse Command Line for 
  for (int i = 1; i < argc; ++i) {
    //cout<<"Command line parameter "<<i<<" is "<<argv[i]<<endl;
    //
    // check if this is the flag for a file with a list of root files
    if (!strcmp(argv[i],"-inlist")) {
      // 
      if (argc < i+1) {
	cout << "<Error> Missing file-list name!" << endl;
	return 1;
      }
      iLOList = i+1;
      ++i;
      useLOList = true;
    } 
    else if (!strcmp(argv[i],"-infile")) {  // Get name of file
      // 
      // check if this is a single root file
      if (argc < i+1) {
	cout << "<Error> Missing input-file name!" << endl;
	return 1;
      }//if
      iLOFile = i+1;
      ++i;
      useLOFile = true;
    }//elseif this is a single file
    else if (!strcmp(argv[i],"-mtop")) {  // Set tarGet top mass for best jet algo
      //
      Double_t mass=atof(argv[i+1]);
      SetTarGetTopMass(mass);
      ++i;
    } //elseif Set top mass
    else if (!strcmp(argv[i],"-lepton")) {  // only process events with positively charge lepton
      //
      cout<<"<Driver> Only processing lepton with Abs PDG ID "<<argv[i+1]<<endl;
      if (argc < i+1) {
	cout << "<Error> Missing Value for lepton flavor -lepton!" << endl;
	return 1;
      }
      SetLeptonSelect(atoi(argv[i+1]));
      ++i;
    } //elseif only specific lepton flavor
    else if (!strcmp(argv[i],"-norm")) {  // only process events with positively charge lepton
      //
      cout<<"<Driver> Additional histogram weight: "<<argv[i+1]<<endl;
      if (argc < i+1) {
	cout << "<Error> Missing Value for histogram weight -norm!" << endl;
	return 1;
      }
      SetNormalization(atof(argv[i+1]));
      ++i;
    } //elseif only positive top
    else if (!strcmp(argv[i],"-num")) {  // number of events
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for number of events -num!" << endl;
	return 1;
      }
      _numEvents=atoi(argv[i+1])+evtOffSet;
      ++i;
    } //elseif num events
    else if (!strcmp(argv[i],"-offSet")) {  // number of events
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for number of events -offSet!" << endl;
	return 1;
      }
      evtOffSet=atoi(argv[i+1]);
      if(_numEvents>0) _numEvents+=evtOffSet;
      ++i;
    } //elseif offSet events
    else if (!strcmp(argv[i],"-hfile")) {  // number of events
      //
      if (argc < i+1) {
	cout << "<Error> Missing Value for histogram filename -hfile!" << endl;
	return 1;
      }
      histoFileName=argv[i+1];
      ++i;
    } //elseif histogram file name
  }//for loop over command line arguments
  
  bool doPathWarning=true;
  //////////////////////////////////////////////
  //
  // Make the chain for the input files.  Use filelist if we have one, else add input file
  // to chain, else exit
  if (useLOList) // Chain files from input filelist
    {
      // Use fstream
      ifstream in(argv[iLOList]);
      int nfiles = 0;
      
      if (in) { // file with list exists
	string fileName,fileOnly,pathOnly;
	int ichain;
	
	cout << "<analysis_main> Chaining files listed in ";
	cout << argv[iLOList] << ":" << endl;
	while (getline(in, fileName)) {
	  cout << "<analysis_main> " << fileName.c_str() << endl;
	  // we have to do some gymnastics here, 
	  // otherwise loading the tree will produce an error:
	  // strip off the path first
	  string::size_type pos = fileName.rfind("/");
	  fileOnly=fileName.substr(pos+1);
	  pathOnly=fileName.substr(0,pos);
	  if(doPathWarning) {
	    cout<<"<analysis_main> Note that these trees don't load properly if they aren't all "<<endl;
	    cout<<"in the same directory.We need to switch into that directory now and then start"<<endl;
	    cout<<"reading things in."<<endl;
	    cout<<"found path: "<<pathOnly<<endl;
	    cout<<"and file "<<fileOnly<<endl;
	    doPathWarning=false;
	  }
	  gSystem->cd(pathOnly.c_str());
	  ichain=coch->Add(fileOnly.c_str());
	  evch->Add(fileOnly.c_str());
	  trch->Add(fileOnly.c_str());
	  if (ichain) ++nfiles;
	}
	cout << "<analysis_main> " << nfiles << " files successfully chained." ;
	cout << endl;
	if (nfiles==0) {
	  cout << "Exiting." << endl;
	  return 1;
	}
      }//if
   } 
  else if (useLOFile) { // Run on input file
    int ichain;
    cout << "<analysis_main> Will run on " << argv[iLOFile] << endl;
    ichain=coch->Add(argv[iLOFile]);
    evch->Add(argv[iLOFile]);
    trch->Add(argv[iLOFile]);
    if (!ichain) {
      cout << "<analysis_main> Error chaining " << argv[iLOFile];
      cout << "! Exiting." << endl;
      return 1;
    }
  } 
  else {
    // Here we don't have any inputs so exit with error.
    cout << "<analysis_main> Need input LO root file/filelist." << endl;
    return 1;
  }

  //
  // Create the root trees
  if(_debugLevel>2) cout<<"Creating analysis classes"<<endl;
  treeCollection=new CollectionTree(coch);
  treeEventTree=new EventTree(evch);
  treeTruthTree=new TruthTree(trch);

  //
  // and make them friends
  if(_debugLevel>5) cout<<"Adding EV0 as a friend tree"<<endl;
  coch->AddFriend(evch,"AliasForEV0");
  if(_debugLevel>5) cout<<"Adding Truth0 as a friend tree"<<endl;
  coch->AddFriend(trch,"AliasForTruth0");


//   //
//   // disable a bunch of branches here if you feel like it
  // better not disable anything in coch. That's so small anyway.
//   evch->SetBranchStatus("*",0);
//   trch->SetBranchStatus("*",0);
//   // and now enable a select few branches
//   evch->SetBranchStatus("El_E",1);
//   evch->SetBranchStatus("El_N",1);
//   evch->SetBranchStatus("El_p_T",1);
//   evch->SetBranchStatus("El_eta",1);
//   evch->SetBranchStatus("El_phi",1);
//   evch->SetBranchStatus("El_EoverP",1);
//   evch->SetBranchStatus("El_charge",1);
//   //evch->SetBranchStatus("",1);
//   evch->SetBranchStatus("PJet_N",1);


  //
  // go back to the right directory before creating the output file
  //gSystem->cd(cwd.c_str());

  
  //
  // open the output histogram file
  if(_debugLevel>0) cout<<"Opening output histogram file"<<histoFileName<<endl;
  openHistogramFile(cwd+"/"+histoFileName);
  
  // 
  // the chains were read in successfully. Now initialize the analysis classes
  Particles::initialize(treeCollection,treeEventTree,treeTruthTree);
  Particles::SetEventCount(evtOffSet);
  return 0;
}
